@@ -6,20 +6,20 @@ from .globalconf import datadir, DATA_DIR
 # server stuff
 from bottle import Bottle, route, get, post, error, run, template, static_file, request, response, FormsDict, redirect, template, HTTPResponse, BaseRequest, abort
 import waitress
-# templating
-from jinja2 import Environment, PackageLoader, select_autoescape
+
 # monkey patching
 from . import monkey
 # rest of the project
 from . import database
-from . import htmlmodules
-from . import htmlgenerators
 from . import malojatime
 from . import utilities
+from . import malojauri
 from .utilities import resolveImage
-from .urihandler import uri_to_internal, remove_identical
+from .urihandler import remove_identical
+from .malojauri import uri_to_internal
 from . import urihandler
 from . import globalconf
+from .jinjaenv.context import jinja_environment
 # doreah toolkit
 from doreah import settings
 from doreah.logging import log
@@ -50,8 +50,7 @@ HOST = settings.get_settings("HOST")
 THREADS = 24
 BaseRequest.MEMFILE_MAX = 15 * 1024 * 1024
 
-WEBFOLDER = pkg_resources.resource_filename(__name__,"web")
-STATICFOLDER = pkg_resources.resource_filename(__name__,"static")
+STATICFOLDER = pkg_resources.resource_filename(__name__,"web/static")
 DATAFOLDER = DATA_DIR
 
 webserver = Bottle()
@@ -91,10 +90,14 @@ def mainpage():
 @webserver.error(500)
 @webserver.error(505)
 def customerror(error):
-	code = int(str(error).split(",")[0][1:])
+	errorcode = error.status_code
+	errordesc = error.status
+	traceback = error.traceback.strip()
 
-	template = jinjaenv.get_template('error.jinja')
-	res = template.render(errorcode=code)
+	adminmode = request.cookies.get("adminmode") == "true" and auth.check(request)
+
+	template = jinja_environment.get_template('error.jinja')
+	res = template.render(errorcode=errorcode,errordesc=errordesc,traceback=traceback,adminmode=adminmode)
 	return res
 
 
@@ -113,7 +116,7 @@ def graceful_exit(sig=None,frame=None):
 @webserver.route("/image")
 def dynamic_image():
 	keys = FormsDict.decode(request.query)
-	relevant, _, _, _ = uri_to_internal(keys)
+	relevant, _, _, _, _ = uri_to_internal(keys)
 	result = resolveImage(**relevant)
 	if result == "": return ""
 	redirect(result,307)
@@ -155,6 +158,7 @@ def static_image(pth):
 @webserver.route("/style.css")
 def get_css():
 	response.content_type = 'text/css'
+	if settings.get_settings("DEV_MODE"): css = generate_css()
 	return css
 
 
@@ -187,43 +191,6 @@ aliases = {
 
 
 
-from . import database_packed
-dbp = database_packed.DB()
-
-JINJA_CONTEXT = {
-	# maloja
-	"db": database,
-	"dbp":dbp,
-	"htmlmodules": htmlmodules,
-	"htmlgenerators": htmlgenerators,
-	"malojatime": malojatime,
-	"utilities": utilities,
-	"urihandler": urihandler,
-	"settings": settings.get_settings,
-	# external
-	"urllib": urllib,
-	"math":math,
-	# config
-	"ranges": [
-		('day','7 days',malojatime.today().next(-6),'day',7),
-		('week','12 weeks',malojatime.thisweek().next(-11),'week',12),
-		('month','12 months',malojatime.thismonth().next(-11),'month',12),
-		('year','10 years',malojatime.thisyear().next(-9),'year',12)
-	],
-	"xranges": [
-		{"identifier":"day","localisation":"14 days","firstrange":malojatime.today().next(-13),"amount":14},
-		{"identifier":"week","localisation":"14 weeks","firstrange":malojatime.thisweek().next(-13),"amount":14},
-		{"identifier":"month","localisation":"14 months","firstrange":malojatime.thismonth().next(-13),"amount":14},
-		{"identifier":"year","localisation":"14 years","firstrange":malojatime.thisyear().next(-13),"amount":14}
-	]
-}
-
-
-jinjaenv = Environment(
-	loader=PackageLoader('maloja', 'web/jinja'),
-	autoescape=select_autoescape(['html', 'xml'])
-)
-jinjaenv.globals.update(JINJA_CONTEXT)
 
 
 @webserver.route("/<name:re:admin.*>")
@@ -240,77 +207,26 @@ def static_html(name):
 	linkheaders = ["</style.css>; rel=preload; as=style"]
 	keys = remove_identical(FormsDict.decode(request.query))
 
-	html_file = os.path.exists(pthjoin(WEBFOLDER,name + ".html"))
-	jinja_file = os.path.exists(pthjoin(WEBFOLDER,"jinja",name + ".jinja"))
-	jinja_pref = settings.get_settings("USE_JINJA")
-
 	adminmode = request.cookies.get("adminmode") == "true" and auth.check(request)
 
 	clock = Clock()
 	clock.start()
 
-	# if a jinja file exists, use this
-	if ("pyhtml" not in keys and jinja_file and jinja_pref) or (jinja_file and not html_file):
-		LOCAL_CONTEXT = {
-			"adminmode":adminmode,
-			"apikey":request.cookies.get("apikey") if adminmode else None,
-			"_urikeys":keys, #temporary!
-		}
-		LOCAL_CONTEXT["filterkeys"], LOCAL_CONTEXT["limitkeys"], LOCAL_CONTEXT["delimitkeys"], LOCAL_CONTEXT["amountkeys"] = uri_to_internal(keys)
+	LOCAL_CONTEXT = {
+		"adminmode":adminmode,
+		"apikey":request.cookies.get("apikey") if adminmode else None,
+		"_urikeys":keys, #temporary!
+	}
+	lc = LOCAL_CONTEXT
+	lc["filterkeys"], lc["limitkeys"], lc["delimitkeys"], lc["amountkeys"], lc["specialkeys"] = uri_to_internal(keys)
 
-		template = jinjaenv.get_template(name + '.jinja')
+	template = jinja_environment.get_template(name + '.jinja')
+	res = template.render(**LOCAL_CONTEXT)
 
-		res = template.render(**LOCAL_CONTEXT)
-		log("Generated page {name} in {time:.5f}s (Jinja)".format(name=name,time=clock.stop()),module="debug_performance")
-		return res
+	if settings.get_settings("DEV_MODE"): jinja_environment.cache.clear()
 
-
-	# if not, use the old way
-	else:
-		try:
-			with open(pthjoin(WEBFOLDER,name + ".html")) as htmlfile:
-				html = htmlfile.read()
-
-			# apply global substitutions
-			with open(pthjoin(WEBFOLDER,"common/footer.html")) as footerfile:
-				footerhtml = footerfile.read()
-			with open(pthjoin(WEBFOLDER,"common/header.html")) as headerfile:
-				headerhtml = headerfile.read()
-			html = html.replace("</body>",footerhtml + "</body>").replace("</head>",headerhtml + "</head>")
-
-
-			# If a python file exists, it provides the replacement dict for the html file
-			if os.path.exists(pthjoin(WEBFOLDER,name + ".py")):
-				#txt_keys = SourceFileLoader(name,"web/" + name + ".py").load_module().replacedict(keys,DATABASE_PORT)
-				try:
-					module = importlib.import_module(".web." + name,package="maloja")
-					txt_keys,resources = module.instructions(keys)
-				except Exception as e:
-					log("Error in website generation: " + str(sys.exc_info()),module="error")
-					raise
-
-				# add headers for server push
-				for resource in resources:
-					if all(ord(c) < 128 for c in resource["file"]):
-						# we can only put ascii stuff in the http header
-						linkheaders.append("<" + resource["file"] + ">; rel=preload; as=" + resource["type"])
-
-				# apply key substitutions
-				for k in txt_keys:
-					if isinstance(txt_keys[k],list):
-						# if list, we replace each occurence with the next item
-						for element in txt_keys[k]:
-							html = html.replace(k,element,1)
-					else:
-						html = html.replace(k,txt_keys[k])
-
-
-			response.set_header("Link",",".join(linkheaders))
-			log("Generated page {name} in {time:.5f}s (Python+HTML)".format(name=name,time=clock.stop()),module="debug_performance")
-			return html
-
-		except:
-			abort(404, "Page does not exist")
+	log("Generated page {name} in {time:.5f}s (Jinja)".format(name=name,time=clock.stop()),module="debug_performance")
+	return res
 
 
 # Shortlinks
